import {Injectable} from '@angular/core';
import {observable, action, computed} from 'mobx-angular';
import {Router} from '@angular/router';
import {Web3Service} from '../../../integrations/services/web3/web3.service';
import { LOCAL_STORAGE_ADDRESS_KEY} from '../../../integrations/dictionaries/meta-mask.dictionary';
import {UtilsService} from '../services/utils.service';


export interface User {
    address: string;
}

@Injectable({
    providedIn: 'root'
})
export class AuthService {
  @observable user: User;
  constructor(private web3: Web3Service,
              private utilsService: UtilsService,
              private router: Router) {}

  @action async init() {
      this.user = this.getUserLoggedIn;
      if ( this.user) {
          this.setUserLoggedIn(this.user);
          await this.web3.setMetamaskProvider();
      } else {
        this.web3.setInfuraProvider().then();

      }
  }
   @action connectToMetaMask() {
        this.web3.connectToMetaMask().subscribe( res => {
            this.setUserLoggedIn({address: res});
            this.router.navigate(['/']).then();
        });
    }


    @action setUserLoggedIn(user: User) {
        this.user = user;
        localStorage.setItem(LOCAL_STORAGE_ADDRESS_KEY, JSON.stringify(user));
        console.log('saved on localStorage');
    }

    @computed  get getUserLoggedIn(): User {
        if (localStorage.getItem(LOCAL_STORAGE_ADDRESS_KEY)) {
            return  JSON.parse(localStorage.getItem(LOCAL_STORAGE_ADDRESS_KEY));
        } else {
            return  null;
        }
    }

    @action logout() {
        this.removeUser();
        this.web3.setInfuraProvider().then();
    }

    @action removeUser() {
        window.localStorage.removeItem(LOCAL_STORAGE_ADDRESS_KEY);
        this.router.navigate(['/']).then();
        this.user = null;
    }

    @computed get isLogged(): boolean {
        return this.user != null;
    }
}
