import {Injectable, OnDestroy} from '@angular/core';
import {AngularWeb3RecentTransactionsService, IAngularWeb3Transaction} from 'angular-web3-components';
import {Subscription} from 'rxjs';
import {Router} from '@angular/router';
import {FormService} from '../../../services/form.service';
import {AuthService} from '../auth/auth.service';
import {StakeSlpService} from './stake-slp.service';

@Injectable({
    providedIn: 'root'
})
export class TransactionsService implements OnDestroy {
    public transactionsSubscription: Subscription;
    private readonly transactionNamesToMethodMapping = [
        {
            name: 'Approve new allowance for stake SLP',
            service: this.stakeSlpService,
            methods: ['fetchSLPAllowance'],
            routs: ['/stake-now']
        },
        {
            name: 'Stake SLP',
            service: this.stakeSlpService,
            methods: ['fetchSLPBalance', 'fetchStaked', 'fetchAPYXft', 'fetchAPYSushi'],
            routs: ['/stake-now']
        },
        {
            name: 'Withdraw SLP',
            service: this.stakeSlpService,
            methods: ['fetchSLPBalance', 'fetchStaked',  'fetchAPYXft', 'fetchAPYSushi'],
            routs: ['/stake-now']
        }
    ];

    constructor(
        private recentTransactionsService: AngularWeb3RecentTransactionsService,
        private router: Router,
        private authService: AuthService,
        private formService: FormService,
        private stakeSlpService: StakeSlpService
    ) {
    }

    public init() {
        this.transactionsSubscription = this.recentTransactionsService.transactionStatusChange.subscribe((res: IAngularWeb3Transaction) => {
            const transactionMethodData = this.transactionNamesToMethodMapping.find(el => el.name === res.name);
            if (transactionMethodData && transactionMethodData.routs.includes(this.router.url)) {
                for (const method of transactionMethodData.methods) {
                    transactionMethodData.service[method](this.authService.user.address);
                }
            }
        });
    }

    ngOnDestroy() {
        this.transactionsSubscription.unsubscribe();
    }
}
