import {Component, OnDestroy, OnInit} from '@angular/core';
import {FormService} from '../../services/form.service';
import {action, computed} from 'mobx-angular';
import {UtilsService} from '../../core/services/services/utils.service';

@Component({
    selector: 'app-form',
    templateUrl: './form.component.html',
    styleUrls: ['./form.component.scss'],
})
export class FormComponent implements OnInit, OnDestroy {

    public selectedCase = 'STAKE';
    constructor(private featuresService: FormService, public utilsService: UtilsService) {}

    @action ngOnInit() {
        this.featuresService.init();
    }


    @computed get slpBalanceDivider() {
        return this.utilsService.parseAmountDivider(this.featuresService.slpBalance);
    }

    @computed get stakedDivider() {
        return this.utilsService.parseAmountDivider(this.featuresService.staked);
    }

    @computed get slpBalance() {
        return this.featuresService.slpBalance;
    }
    @computed get slpAllowance() {
        return this.featuresService.slpAllowance;
    }
    @computed get staked() {
        return this.featuresService.staked;
    }

    @computed get formStake() {
        return this.featuresService.formStake;
    }
    @computed get apySushi() {
        return this.featuresService.apySushi;
    }
    @computed get apyXFT() {
        return this.featuresService.apyXFT;
    }

    @computed get formUnstake() {
        return this.featuresService.formUnstake;
    }
    @computed get isSubmittingApprove() {
        return this.featuresService.isSubmittingApprove;
    }
    @computed get isSubmittingStake() {
        return this.featuresService.isSubmittingStake;
    }
    @computed get isSubmittingUnstake() {
        return this.featuresService.isSubmittingUnstake;
    }
    @computed get isSubmittingClaimRewards() {
        return this.featuresService.isSubmittingClaimRewards;
    }

    @computed get rewardsXFT() {
        return this.featuresService.rewardsXFT;
    }
    @computed get rewardsSushi() {
        return this.featuresService.rewardsSushi;
    }
    @computed get stakeAmount() {
        return this.featuresService.stakeAmount;
    }
   @computed get unstakeAmount() {
        return this.featuresService.unstakeAmount;
    }

    @action submitFormStake() {
        this.featuresService.submitFormStake();
    }

    @action submitFormUnstake() {
        this.featuresService.submitFormUnstake();
    }
    @action submitClaimRewards() {
        this.featuresService.submitClaimRewards();
    }

    public lessOrEqual(value1: string, value2: string): boolean {
        return this.utilsService.lessOrEqual(value1, value2);
}
    public greaterThen(value1: string, value2: string): boolean {
        return this.utilsService.greaterThen(value1, value2);
    }

    public ngOnDestroy(): void {
    this.featuresService.unsubscribe();
    }


}
